# ----------------------------------------------------------------------------
# Copyright 2017 Nervana Systems Inc.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ----------------------------------------------------------------------------
from contextlib import contextmanager
import shelve

from filelock import FileLock


_locks = {}


def _lock(filename):
    try:
        return _locks[filename]
    except KeyError:
        _locks[filename] = FileLock(filename + '.lock')
    return _locks[filename]


@contextmanager
def atomic_shelve(filename):
    with _lock(filename):
        shelved = shelve.open(filename)
        yield shelved
        shelved.close()
